@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed.c,v 1.65.4.1 2005/10/02 18:49:57 chpe Exp $
+ *  $Id: ephy-embed.c,v 1.70 2005/12/07 17:54:56 chpe Exp $
  */
 
 #include "config.h"
@@ -79,6 +79,7 @@ ephy_embed_base_init (gpointer g_class)
  * EphyEmbed::ge-popup-blocked:
  * @embed:
  * @address: The requested URL
+ * @target: The requested window name, e.g. "_blank"
  * @features: The requested features: for example, "height=400,width=200"
  *
  * The ::ge_popup_blocked signal is emitted when the viewed web page requests
@@ -89,9 +90,10 @@ ephy_embed_base_init (gpointer g_class)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EphyEmbedIface, popup_blocked),
 			      NULL, NULL,
-			      ephy_marshal_VOID__STRING_STRING,
+			      ephy_marshal_VOID__STRING_STRING_STRING,
 			      G_TYPE_NONE,
-			      2,
+			      3,
+			      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
 			      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
 			      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
 /**
@@ -642,6 +644,59 @@ ephy_embed_get_zoom (EphyEmbed *embed)
 	return iface->get_zoom (embed);
 }
 
+/**
+ * ephy_embed_scroll:
+ * @embed: an #EphyEmbed
+ * @num_lines: The number of lines to scroll by
+ *
+ * Scrolls the view by lines. Positive numbers scroll down, negative
+ * numbers scroll up
+ *
+ **/
+void
+ephy_embed_scroll (EphyEmbed *embed,
+		   int num_lines)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
+	iface->scroll_lines (embed, num_lines);
+}
+
+/**
+ * ephy_embed_page_scroll:
+ * @embed: an #EphyEmbed
+ * @num_lines: The number of pages to scroll by
+ *
+ * Scrolls the view by pages. Positive numbers scroll down, negative
+ * numbers scroll up
+ *
+ **/
+void
+ephy_embed_page_scroll (EphyEmbed *embed,
+			int num_pages)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
+	iface->scroll_pages (embed, num_pages);
+}
+
+/**
+ * ephy_embed_scroll_pixels:
+ * @embed: an #EphyEmbed
+ * @dx: the number of pixels to scroll in X direction
+ * @dy: the number of pixels to scroll in Y direction
+ *
+ * Scrolls the view by pixels. Positive numbers scroll down resp. right,
+ * negative numbers scroll up resp. left.
+ *
+ **/
+void
+ephy_embed_scroll_pixels (EphyEmbed *embed,
+			  int dx,
+			  int dy)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
+	iface->scroll_pixels (embed, dx, dy);
+}
+
 /**
  * ephy_embed_shistory_n_items:
  * @embed: an #EphyEmbed
@@ -717,6 +772,28 @@ ephy_embed_shistory_go_nth (EphyEmbed *embed,
 	iface->shistory_go_nth (embed, nth);
 }
 
+/**
+ * ephy_embed_shistory_copy:
+ * @source: the #EphyEmbed to copy the history from
+ * @dest: the #EphyEmbed to copy the history to
+ * @copy_back: %TRUE to copy the back history
+ * @copy_forward: %TRUE to copy the forward history
+ * @copy_current: %TRUE to set the current page to that in the copied history
+ *
+ * Copy's the back and/or forward history from @source to @dest,
+ * and optionally set @dest to the current page of @source as well.
+ **/
+void
+ephy_embed_shistory_copy (EphyEmbed *source,
+			  EphyEmbed *dest,
+			  gboolean copy_back,
+			  gboolean copy_forward,
+			  gboolean copy_current)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (source);
+	iface->shistory_copy (source, dest, copy_back, copy_forward, copy_current);
+}
+
 /**
  * ephy_embed_get_security_level:
  * @embed: an #EphyEmbed
