@@ -1,6 +1,8 @@
 /*  vim:set ts=8 noet sw=8:
  *  Copyright (C) 2000-2004 Marco Pesenti Gritti
+ *  Copyright (C) 2003 Robert Marcano
  *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2005 Crispin Flowerday
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-single.cpp,v 1.114 2005/07/24 21:37:01 chpe Exp $
+ *  $Id: mozilla-embed-single.cpp,v 1.121 2005/12/11 14:40:15 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -28,6 +30,7 @@
 #include "ephy-cookie-manager.h"
 #include "ephy-password-manager.h"
 #include "ephy-permission-manager.h"
+#include "ephy-certificate-manager.h"
 #include "ephy-embed-shell.h"
 
 #include "glib.h"
@@ -44,6 +47,7 @@
 #include "EphyBrowser.h"
 #include "EphyUtils.h"
 #include "MozillaPrivate.h"
+#include "mozilla-x509-cert.h"
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -68,6 +72,11 @@
 #include <nsILocalFile.h>
 #include <nsIURI.h>
 
+#ifdef HAVE_MOZILLA_PSM
+#include <nsIX509Cert.h>
+#include <nsIX509CertDB.h>
+#endif
+
 #ifdef HAVE_NSIPASSWORD_H
 #include <nsIPassword.h>
 #endif
@@ -105,6 +114,14 @@ struct MozillaEmbedSinglePrivate
 	GtkWidget *theme_window;
 
 	EphySingle *mSingleObserver;
+
+	guint online : 1;
+};
+
+enum
+{
+	PROP_0,
+	PROP_NETWORK_STATUS
 };
 
 static void mozilla_embed_single_class_init	(MozillaEmbedSingleClass *klass);
@@ -113,7 +130,10 @@ static void ephy_cookie_manager_iface_init	(EphyCookieManagerIface *iface);
 static void ephy_password_manager_iface_init	(EphyPasswordManagerIface *iface);
 static void ephy_permission_manager_iface_init	(EphyPermissionManagerIface *iface);
 static void mozilla_embed_single_init		(MozillaEmbedSingle *ges);
-static gboolean have_gnome_url_handler		(const gchar *protocol);
+
+#ifdef ENABLE_CERTIFICATE_MANAGER
+static void ephy_certificate_manager_iface_init	(EphyCertificateManagerIface *iface);
+#endif
 
 static GObjectClass *parent_class = NULL;
 
@@ -165,6 +185,15 @@ mozilla_embed_single_get_type (void)
 			NULL
 		};
 
+#ifdef ENABLE_CERTIFICATE_MANAGER
+		static const GInterfaceInfo certificate_manager_info =
+		{
+			(GInterfaceInitFunc) ephy_certificate_manager_iface_init,
+			NULL,
+			NULL
+		};
+#endif
+
 		type = g_type_register_static (G_TYPE_OBJECT,
 					       "MozillaEmbedSingle",
 					       &our_info,
@@ -185,6 +214,11 @@ mozilla_embed_single_get_type (void)
 		g_type_add_interface_static (type,
 					     EPHY_TYPE_PERMISSION_MANAGER,
 					     &permission_manager_info);
+#ifdef ENABLE_CERTIFICATE_MANAGER
+		g_type_add_interface_static (type,
+					     EPHY_TYPE_CERTIFICATE_MANAGER,
+					     &certificate_manager_info);
+#endif
 	}
 
         return type;
@@ -250,8 +284,7 @@ mozilla_set_default_prefs (MozillaEmbedSingle *mes)
 #endif
 
 	/* Open ftp uris with an external handler if one is setup */
-	pref->SetBoolPref("network.protocol-handler.external.ftp",
-			  have_gnome_url_handler ("ftp"));
+	pref->SetBoolPref ("network.protocol-handler.external.ftp", PR_FALSE);	
 
 	return TRUE;
 }
@@ -358,8 +391,11 @@ mozilla_init_plugin_path ()
 	user_path = g_getenv ("MOZ_PLUGIN_PATH");
 	new_path = g_strconcat (user_path ? user_path : "",
 				user_path ? ":" : "",
-				MOZILLA_PREFIX "/lib/mozilla/plugins:"
-				MOZILLA_HOME "/plugins",
+				MOZILLA_PREFIX "/lib/mozilla/plugins"
+				":" MOZILLA_HOME "/plugins",
+#ifdef HAVE_PRIVATE_PLUGINS
+				":" PLUGINDIR,
+#endif
 				NULL);
 
 	g_setenv ("MOZ_PLUGIN_PATH", new_path, TRUE);
@@ -404,30 +440,6 @@ mozilla_init_profile (void)
         g_free (profile_path);
 }
 
-static gboolean
-have_gnome_url_handler (const gchar *protocol)
-{
-	gchar *key, *cmd;
-	gboolean rv;
-
-	key = g_strdup_printf ("/desktop/gnome/url-handlers/%s/command", 
-			       protocol);
-	cmd = eel_gconf_get_string (key);
-	g_free (key);
-
-	rv = (cmd != NULL && strstr (cmd, "epiphany") == NULL);
-	g_free (cmd);
-
-	if (!rv) return rv;
-
-	key = g_strdup_printf ("/desktop/gnome/url-handlers/%s/enabled", 
-			       protocol);
-	rv = eel_gconf_get_boolean (key);
-	g_free (key);
-
-	return rv;
-}
-
 #if defined(MOZ_NSIXULCHROMEREGISTRY_SELECTSKIN) || defined(HAVE_CHROME_NSICHROMEREGISTRYSEA_H)
 static nsresult
 getUILang (nsAString& aUILang)
@@ -649,18 +661,23 @@ impl_clear_auth_cache (EphyEmbedSingle *shell)
 }
 
 static void
-impl_set_offline_mode (EphyEmbedSingle *shell,
-		       gboolean offline)
+impl_set_network_status (EphyEmbedSingle *single,
+			 gboolean online)
 {
 	nsCOMPtr<nsIIOService> io = do_GetService(NS_IOSERVICE_CONTRACTID);
 	if (!io) return;
 
-	io->SetOffline(offline);
+	io->SetOffline (!online);
 }
 
 static gboolean
-impl_get_offline_mode (EphyEmbedSingle *shell)
+impl_get_network_status (EphyEmbedSingle *esingle)
 {
+	MozillaEmbedSingle *single = MOZILLA_EMBED_SINGLE (esingle);
+	MozillaEmbedSinglePrivate *priv = single->priv;
+
+	NS_ENSURE_TRUE (priv->mSingleObserver, TRUE);
+
 	nsCOMPtr<nsIIOService> io = do_GetService(NS_IOSERVICE_CONTRACTID);
 	if (!io) return FALSE; /* no way to check the state, assume offline */
 
@@ -669,7 +686,12 @@ impl_get_offline_mode (EphyEmbedSingle *shell)
 	rv = io->GetOffline(&isOffline);
 	NS_ENSURE_SUCCESS (rv, FALSE);
 
-	return isOffline;
+	PRBool isOnline = !isOffline;
+	PRBool reallyOnline = priv->mSingleObserver->IsOnline ();
+
+	g_return_val_if_fail (reallyOnline == isOnline, TRUE);
+
+	return !isOffline;
 }
 
 static GList *
@@ -815,11 +837,18 @@ impl_list_passwords (EphyPasswordManager *manager)
 		NS_UTF16ToCString (unicodeName,
 				   NS_CSTRING_ENCODING_UTF8, userName);
 
+		rv = nsPassword->GetPassword (unicodeName);
+		if (NS_FAILED (rv)) continue;
+
+		nsEmbedCString userPassword;
+		NS_UTF16ToCString (unicodeName,
+				   NS_CSTRING_ENCODING_UTF8, userPassword);
+
 		EphyPasswordInfo *p = g_new0 (EphyPasswordInfo, 1);
 
 		p->host = g_strdup (host.get());
 		p->username = g_strdup (userName.get());
-		p->password = NULL;
+		p->password = g_strdup (userPassword.get());
 
 		passwords = g_list_prepend (passwords, p);
 	}
@@ -1007,6 +1036,155 @@ impl_open_window (EphyEmbedSingle *single,
 	return EphyUtils::FindEmbed (newWindow);
 }
 
+#ifdef ENABLE_CERTIFICATE_MANAGER
+
+static gboolean
+impl_remove_certificate (EphyCertificateManager *manager,
+			 EphyX509Cert *cert)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIX509CertDB> certDB;
+	certDB = do_GetService (NS_X509CERTDB_CONTRACTID);
+	if (!certDB) return FALSE;
+
+        nsCOMPtr<nsIX509Cert> mozCert;
+        rv = mozilla_x509_cert_get_mozilla_cert (MOZILLA_X509_CERT (cert),
+						 getter_AddRefs (mozCert));
+	if (NS_FAILED (rv)) return FALSE;
+
+	rv = certDB->DeleteCertificate (mozCert);
+	if (NS_FAILED (rv)) return FALSE;
+
+        return TRUE;
+}
+
+#define NICK_DELIMITER PRUnichar('\001')
+
+static GList *
+retrieveCerts (PRUint32 type)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIX509CertDB> certDB;
+	certDB = do_GetService (NS_X509CERTDB_CONTRACTID);
+	if (!certDB) return NULL;
+
+	PRUint32 count;
+	PRUnichar **certNameList = NULL;
+
+	rv = certDB->FindCertNicknames (NULL, type, &count, &certNameList);
+	if (NS_FAILED (rv)) return NULL;
+
+	LOG("Certificates found: %i", count);
+
+	GList *list = NULL;
+	for (PRUint32 i = 0; i < count; i++)
+	{
+		/* HACK HACK, this is EVIL, the string for each cert is:
+		     <DELIMITER>nicknameOrEmailAddress<DELIMITER>dbKey
+		   So we need to chop off the dbKey to look it up in the database.
+		   
+		   https://bugzilla.mozilla.org/show_bug.cgi?id=214742
+		*/
+		nsEmbedCString full_string;
+		NS_UTF16ToCString (nsEmbedString(certNameList[i]),
+				   NS_CSTRING_ENCODING_UTF8, full_string);
+
+		const char *key = full_string.get();
+		char *pos = strrchr (key, NICK_DELIMITER);
+		if (!pos) continue;
+
+		nsCOMPtr<nsIX509Cert> mozilla_cert;
+		rv = certDB->FindCertByDBKey (pos, NULL, getter_AddRefs (mozilla_cert));
+		if (NS_FAILED (rv)) continue;
+
+		MozillaX509Cert *cert = mozilla_x509_cert_new (mozilla_cert);
+		list = g_list_prepend (list, cert);
+	}
+
+	NS_FREE_XPCOM_ALLOCATED_POINTER_ARRAY (count, certNameList);
+	return list;
+}
+
+static GList *
+impl_get_certificates (EphyCertificateManager *manager,
+		       EphyX509CertType type)
+{
+	int moz_type = nsIX509Cert::USER_CERT;
+	switch (type)
+	{
+		case PERSONAL_CERTIFICATE:
+			moz_type = nsIX509Cert::USER_CERT;
+			break;
+		case SERVER_CERTIFICATE:
+			moz_type = nsIX509Cert::SERVER_CERT;
+			break;
+		case CA_CERTIFICATE:
+			moz_type = nsIX509Cert::CA_CERT;
+			break;
+	}
+	return retrieveCerts (moz_type);
+}
+
+static gboolean
+impl_import (EphyCertificateManager *manager,
+	     const gchar *file)
+{
+	nsresult rv;
+	nsCOMPtr<nsIX509CertDB> certDB;
+	certDB = do_GetService (NS_X509CERTDB_CONTRACTID);
+	if (!certDB) return NULL;
+
+	nsCOMPtr<nsILocalFile> localFile;
+	localFile = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
+
+	// TODO Is this correct ?
+	nsEmbedString path;
+	NS_CStringToUTF16 (nsEmbedCString(file),
+			   NS_CSTRING_ENCODING_UTF8, path);
+
+
+	localFile->InitWithPath (path);
+	rv = certDB->ImportPKCS12File(NULL, localFile);
+	if (NS_FAILED (rv)) return FALSE;
+
+	return TRUE;
+}
+
+#endif /* ENABLE_CERTIFICATE_MANAGER */
+
+static void
+mozilla_embed_single_get_property (GObject *object,
+				   guint prop_id,
+				   GValue *value,
+				   GParamSpec *pspec)
+{
+	EphyEmbedSingle *single = EPHY_EMBED_SINGLE (object);
+
+	switch (prop_id)
+	{
+		case PROP_NETWORK_STATUS:
+			g_value_set_boolean (value, ephy_embed_single_get_network_status (single));
+			break;
+	}
+}
+
+static void
+mozilla_embed_single_set_property (GObject *object,
+				   guint prop_id,
+				   const GValue *value,
+				   GParamSpec *pspec)
+{
+	EphyEmbedSingle *single = EPHY_EMBED_SINGLE (object);
+
+	switch (prop_id)
+	{
+		case PROP_NETWORK_STATUS:
+			ephy_embed_single_set_network_status (single, g_value_get_boolean (value));
+			break;
+	}
+}
 static void
 mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
 {
@@ -1016,6 +1194,10 @@ mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
 
 	object_class->dispose = mozilla_embed_single_dispose;
 	object_class->finalize = mozilla_embed_single_finalize;
+	object_class->get_property = mozilla_embed_single_get_property;
+	object_class->set_property = mozilla_embed_single_set_property;
+
+	g_object_class_override_property (object_class, PROP_NETWORK_STATUS, "network-status");
 
 	g_type_class_add_private (object_class, sizeof (MozillaEmbedSinglePrivate));
 }
@@ -1025,8 +1207,8 @@ ephy_embed_single_iface_init (EphyEmbedSingleIface *iface)
 {
 	iface->clear_cache = impl_clear_cache;
 	iface->clear_auth_cache = impl_clear_auth_cache;
-	iface->set_offline_mode = impl_set_offline_mode;
-	iface->get_offline_mode = impl_get_offline_mode;
+	iface->set_network_status = impl_set_network_status;
+	iface->get_network_status = impl_get_network_status;
 	iface->get_font_list = impl_get_font_list;
 	iface->open_window = impl_open_window;
 }
@@ -1056,3 +1238,15 @@ ephy_permission_manager_iface_init (EphyPermissionManagerIface *iface)
 	iface->test = impl_permission_manager_test;
 	iface->list = impl_permission_manager_list;
 }
+
+#ifdef ENABLE_CERTIFICATE_MANAGER
+
+static void
+ephy_certificate_manager_iface_init (EphyCertificateManagerIface *iface)
+{
+	iface->get_certificates = impl_get_certificates;
+	iface->remove_certificate = impl_remove_certificate;
+	iface->import = impl_import;
+}
+
+#endif /* ENABLE_CERTIFICATE_MANAGER */
