@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-location-entry.c,v 1.65 2005/10/06 18:28:55 chpe Exp $
+ *  $Id: ephy-location-entry.c,v 1.69 2005/11/10 21:32:33 crispin Exp $
  */
 
 #include "config.h"
@@ -299,14 +299,14 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
                                                                      "Secure background colour",
                                                                      "Background colour to use for secure sites",
                                                                      GDK_TYPE_COLOR,
-                                                                     G_PARAM_READABLE));
+                                                                     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
         gtk_widget_class_install_style_property (widget_class,
                                                  g_param_spec_boxed ("secure-fg-color",
                                                                      "Secure foreground Colour",
                                                                      "Foreground colour to use for secure sites",
                                                                      GDK_TYPE_COLOR,
-                                                                     G_PARAM_READABLE));
+                                                                     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private (object_class, sizeof (EphyLocationEntryPrivate));
 }
@@ -616,6 +616,148 @@ each_url_get_data_binder (EphyDragEachSelectedItemDataGet iteratee,
 	g_free (title);
 }
 
+#define DRAG_ICON_LAYOUT_BORDER		2
+#define DRAG_ICON_ICON_SPACING		DRAG_ICON_LAYOUT_BORDER * 2
+#define DRAG_ICON_MAX_WIDTH_CHARS	32
+
+static GdkPixmap *
+favicon_create_drag_pixmap (EphyLocationEntry *entry,
+			    GtkWidget *widget)
+{
+	EphyLocationEntryPrivate *priv = entry->priv;
+	char *title = NULL, *address = NULL;
+	GString *text;
+	GdkDrawable *drawable;
+	PangoContext *context;
+	PangoLayout  *layout;
+	PangoFontMetrics *metrics;
+	int pixmap_height, pixmap_width;
+	int layout_width, layout_height;
+	int icon_width = 0, icon_height = 0, offset_x = 0;
+	int char_width;
+
+	g_signal_emit (entry, signals[GET_LOCATION], 0, &address);
+	g_signal_emit (entry, signals[GET_TITLE], 0, &title);
+	if (address == NULL || title == NULL) return NULL;
+
+	/* Compute text */
+	title = g_strstrip (title);
+
+	text = g_string_sized_new (strlen (address) + strlen (title) + 2);
+	if (title[0] != '\0')
+	{
+		g_string_append (text, title);
+		g_string_append (text, "\n");
+	}
+
+	if (address[0] != '\0')
+	{
+		g_string_append (text, address);
+	}
+
+	/* Now build the pixmap */
+
+	if (priv->favicon != NULL)
+	{
+		icon_width = gdk_pixbuf_get_width (priv->favicon);
+		icon_height = gdk_pixbuf_get_height (priv->favicon);
+	}
+
+	context = gtk_widget_get_pango_context (widget);
+	layout = pango_layout_new (context);
+
+	context = gtk_widget_get_pango_context (widget);
+	metrics = pango_context_get_metrics (context,
+					     widget->style->font_desc,
+					     pango_context_get_language (context));
+
+	char_width = pango_font_metrics_get_approximate_digit_width (metrics);
+	pango_font_metrics_unref (metrics);
+
+	pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_END);
+	pango_layout_set_width (layout, char_width * DRAG_ICON_MAX_WIDTH_CHARS);
+	pango_layout_set_text (layout, text->str, text->len);
+
+	pango_layout_get_size (layout, &layout_width, &layout_height);
+
+	pixmap_width  = layout_width  / PANGO_SCALE + DRAG_ICON_LAYOUT_BORDER * 2;
+
+	if (priv->favicon != NULL)
+	{
+		offset_x = icon_width + 2 * DRAG_ICON_ICON_SPACING;
+		pixmap_width += offset_x;
+		pixmap_height = MAX (layout_height / PANGO_SCALE, icon_height) + DRAG_ICON_LAYOUT_BORDER * 2;
+	}
+	else
+	{
+		pixmap_height = layout_height / PANGO_SCALE + DRAG_ICON_LAYOUT_BORDER * 2;
+	}
+
+	drawable = gdk_pixmap_new (widget->window,
+				   pixmap_width  + 2,
+				   pixmap_height + 2,
+				   -1);
+
+	gdk_draw_rectangle (drawable,
+			    widget->style->base_gc [GTK_WIDGET_STATE (widget)],
+			    TRUE,
+			    0, 0,
+			    pixmap_width + 1,
+			    pixmap_height + 1);
+
+
+	if (priv->favicon != NULL)
+	{
+		gdk_draw_pixbuf (drawable,
+				 widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+				 priv->favicon,
+				 0, 0, 
+				 1 + DRAG_ICON_LAYOUT_BORDER + DRAG_ICON_ICON_SPACING,
+				 1 + DRAG_ICON_LAYOUT_BORDER + (pixmap_height - icon_height) / 2,
+				 -1, -1,
+				 GDK_RGB_DITHER_NONE, 0, 0);
+	}
+
+	gdk_draw_layout (drawable,
+			 widget->style->text_gc [GTK_WIDGET_STATE (widget)],
+			 1 + DRAG_ICON_LAYOUT_BORDER + offset_x,
+			 1 + DRAG_ICON_LAYOUT_BORDER,
+			 layout);
+
+	gdk_draw_rectangle (drawable,
+			    widget->style->black_gc,
+			    FALSE,
+			    0, 0,
+			    pixmap_width + 1,
+			    pixmap_height + 1);
+
+	g_object_unref (layout);
+
+	g_free (address);
+	g_free (title);
+	g_string_free (text,TRUE);
+
+	return drawable;
+}
+
+static void
+favicon_drag_begin_cb (GtkWidget *widget,
+		       GdkDragContext *context,
+		       EphyLocationEntry *entry)
+{
+	GdkPixmap *pixmap;
+
+	pixmap = favicon_create_drag_pixmap (entry, widget);
+
+	if (pixmap != NULL)
+	{
+		gtk_drag_set_icon_pixmap (context,
+					  gdk_drawable_get_colormap (pixmap),
+					  pixmap, NULL, -2, -2);
+		g_object_unref (pixmap);
+	}
+}
+
 static void
 favicon_drag_data_get_cb (GtkWidget *widget,
 			  GdkDragContext *context,
@@ -665,9 +807,13 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 	priv->icon_ebox = gtk_event_box_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (priv->icon_ebox), 2);
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->icon_ebox), FALSE);
+	gtk_widget_add_events (priv->icon_ebox, GDK_BUTTON_PRESS_MASK |
+			      			GDK_BUTTON_RELEASE_MASK |
+						GDK_POINTER_MOTION_HINT_MASK);
 	gtk_drag_source_set (priv->icon_ebox, GDK_BUTTON1_MASK,
 			     url_drag_types, G_N_ELEMENTS (url_drag_types),
 			     GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
+
 	gtk_tooltips_set_tip (priv->tips, priv->icon_ebox,
 			      _("Drag and drop this icon to create a link to this page"), NULL);
 
@@ -689,6 +835,9 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 
 	g_signal_connect (priv->icon_ebox, "drag-data-get",
 			  G_CALLBACK (favicon_drag_data_get_cb), entry);
+	g_signal_connect (priv->icon_ebox, "drag-begin",
+			  G_CALLBACK (favicon_drag_begin_cb), entry);
+
 	g_signal_connect (priv->lock_ebox, "button-press-event",
 			  G_CALLBACK (lock_button_press_event_cb), entry);
 
