@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-new-bookmark.c,v 1.53 2005/01/08 19:41:47 chpe Exp $
+ *  $Id: ephy-new-bookmark.c,v 1.58 2005/11/08 12:47:17 chpe Exp $
  */
 
 #include "config.h"
@@ -129,14 +129,14 @@ ephy_new_bookmark_class_init (EphyNewBookmarkClass *klass)
 							      "Bookmarks set",
 							      "Bookmarks set",
 							      EPHY_TYPE_BOOKMARKS,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
 					 PROP_LOCATION,
 					 g_param_spec_string ("location",
 							      "Bookmark location",
 							      "Bookmark location",
 							      "",
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (object_class, sizeof(EphyNewBookmarkPrivate));
 }
@@ -309,7 +309,7 @@ duplicate_dialog_construct (GtkWindow *parent,
 	GtkWidget *dialog;
 	char *str, *tmp_str;
 
-	tmp_str = g_markup_printf_escaped (_("You already have a bookmark titled \"%s\" for this page."),
+	tmp_str = g_markup_printf_escaped (_("You already have a bookmark titled “%s” for this page."),
 				   	   title);
 	str = g_strconcat ("<span weight=\"bold\" size=\"larger\">", tmp_str, "</span>", NULL);
 
@@ -466,7 +466,7 @@ ephy_new_bookmark_set_title (EphyNewBookmark *bookmark,
 {
 	const char *real_title;
 
-	LOG ("Setting new bookmark title to: \"%s\"", title)
+	LOG ("Setting new bookmark title to: \"%s\"", title);
 
 	if (title == NULL || strlen (title) == 0)
 	{
