@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-encoding-menu.c,v 1.33 2005/01/28 22:24:45 chpe Exp $
+ *  $Id: ephy-encoding-menu.c,v 1.36 2005/11/08 12:46:43 chpe Exp $
  */
 
 #include "config.h"
@@ -172,7 +172,7 @@ update_encoding_menu_cb (GtkAction *dummy, EphyEncodingMenu *menu)
 	action = gtk_action_group_get_action (p->action_group,
 					      "ViewEncodingAutomatic");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), is_automatic);
-	g_object_set (G_OBJECT (action), "sensitive", !is_automatic, NULL);
+	gtk_action_set_sensitive (action, !is_automatic);
 
 	/* set the encodings group's active member */
 	g_snprintf (name, sizeof (name), "Encoding%s", encoding);
@@ -345,21 +345,19 @@ ephy_encoding_menu_automatic_cb (GtkAction *action, EphyEncodingMenu *menu)
 	ephy_embed_set_encoding (embed, "");
 }
 
-static GtkActionEntry menu_entries [] =
+static const GtkActionEntry menu_entries [] =
 {
 	{ "ViewEncodingOther", NULL, N_("_Other..."), NULL,
 	  N_("Other encodings"),
 	  G_CALLBACK (ephy_encoding_menu_view_dialog_cb) }
 };
-static guint n_menu_entries = G_N_ELEMENTS (menu_entries);
 
-static GtkToggleActionEntry toggle_menu_entries [] =
+static const GtkToggleActionEntry toggle_menu_entries [] =
 {
 	{ "ViewEncodingAutomatic", NULL, N_("_Automatic"), NULL,
 	  N_("Use the encoding specified by the document"),
 	  G_CALLBACK (ephy_encoding_menu_automatic_cb), FALSE }
 };
-static const guint n_toggle_menu_entries = G_N_ELEMENTS (toggle_menu_entries);
 
 static void
 ephy_encoding_menu_set_window (EphyEncodingMenu *menu, EphyWindow *window)
@@ -380,9 +378,9 @@ ephy_encoding_menu_set_window (EphyEncodingMenu *menu, EphyWindow *window)
 	menu->priv->action_group = action_group;
 
 	gtk_action_group_add_actions (action_group, menu_entries,
-				      n_menu_entries, menu);
+				      G_N_ELEMENTS (menu_entries), menu);
 	gtk_action_group_add_toggle_actions (action_group, toggle_menu_entries,
-                                    	     n_toggle_menu_entries, menu);
+                                    	     G_N_ELEMENTS (toggle_menu_entries), menu);
 
 	/* add actions for the existing encodings */
 	encodings = ephy_encodings_get_all (menu->priv->encodings);
@@ -477,7 +475,7 @@ ephy_encoding_menu_class_init (EphyEncodingMenuClass *klass)
 							      "Window",
 							      "Parent window",
 							      EPHY_TYPE_WINDOW,
-							      G_PARAM_READWRITE |
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB |
 							      G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (object_class, sizeof(EphyEncodingMenuPrivate));
