@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: prefs-dialog.c,v 1.118 2005/10/30 22:58:12 chpe Exp $
+ *  $Id: prefs-dialog.c,v 1.119 2005/11/15 13:35:38 chpe Exp $
  */
 
 #include "config.h"
@@ -785,7 +785,7 @@ static EphyDialog *
 setup_add_language_dialog (PrefsDialog *pd)
 {
 	EphyDialog *dialog;
-	GtkWidget *window;
+	GtkWidget *window, *parent;
 	GtkListStore *store;
 	GtkTreeModel *sortmodel;
 	GtkTreeView *treeview;
@@ -795,10 +795,10 @@ setup_add_language_dialog (PrefsDialog *pd)
 	GtkTreeIter iter;
 	int i;
 
-	window = ephy_dialog_get_control (EPHY_DIALOG (pd), properties[WINDOW_PROP].id);
+	parent = ephy_dialog_get_control (EPHY_DIALOG (pd), properties[WINDOW_PROP].id);
 
 	dialog =  EPHY_DIALOG (g_object_new (EPHY_TYPE_DIALOG,
-					     "parent-window", window,
+					     "parent-window", parent,
 					     "default-width", 260,
 					     "default-height", 230,
 					     NULL));
@@ -837,6 +837,9 @@ setup_add_language_dialog (PrefsDialog *pd)
 				  add_lang_props[LANGUAGE_DIALOG].id, &window,
 				  NULL);
 
+	gtk_window_group_add_window (GTK_WINDOW (parent)->group, GTK_WINDOW (window));
+	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
+
 	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview), FALSE);
 
 	gtk_tree_view_set_model (treeview, sortmodel);
@@ -1272,6 +1275,9 @@ prefs_dialog_init (PrefsDialog *pd)
 		 properties[CLEAR_CACHE_BUTTON_PROP].id, &clear_cache_button,
 		 NULL);
 
+	ephy_gui_ensure_window_group (GTK_WINDOW (window));
+
+	gtk_window_set_role (GTK_WINDOW (window), "epiphany-preferences");
 	gtk_window_set_icon_name (GTK_WINDOW (window), GTK_STOCK_PREFERENCES);
 
 	g_signal_connect (window, "response",
